#pragma once
#include <fstream>
#include <set>
#include "Exception.h"
#include <iostream>

class TravelCompany {
private:
	struct Data {
		std::string tour_name;
		std::string surname;
		int price;
		int days;
		int moving_cost;
		int exchange_rate;
		int amount;
		int travel_cost;

		Data() :
			tour_name(""),
			surname(""),
			price(0.0f),
			days(0),
			moving_cost(0.0f),
			exchange_rate(0.0f),
			amount(0.0f),
			travel_cost(0.0f) 
		{}

		Data(std::string tour_name,
			std::string surname,
			int price,
			int days,
			int moving_cost,
			int exchange_rate,
			int amount,
			int travel_cost) :
			tour_name(tour_name),
			surname(surname),
			price(price),
			days(days),
			moving_cost(moving_cost),
			exchange_rate(exchange_rate),
			amount(amount),
			travel_cost(travel_cost)
		{}


		bool operator == (const Data& data) {
			return (this->tour_name == data.tour_name &&
				this->surname == data.surname &&
				this->price == data.price &&
				this->days == data.days &&
				this->moving_cost == data.moving_cost &&
				this->exchange_rate == data.exchange_rate &&
				this->amount == data.amount &&
				this->travel_cost == data.travel_cost);
		}
		bool operator != (const Data& data) {
			return !(*this == data);
		}

		friend bool operator < (const Data& data, const Data& data2) {
			return data.travel_cost < data2.travel_cost;
		}

		bool in(const std::multiset<Data> cont) {
			for (auto i = cont.begin(); i != cont.end(); ++i) {
				if (*this == *i)
					return true;
			}
			return false;
		}

	};

	std::multiset<Data> cont;

public:

	TravelCompany() : cont(std::multiset<Data>()) {}

	TravelCompany(std::multiset<Data> cont) : cont(cont) {}

	bool operator == (const TravelCompany&);

	bool operator != (const TravelCompany& data) {
		return !(*this == data);
	}

	friend std::ostream& operator << (std::ostream&, const TravelCompany&);
	friend std::istream& operator >> (std::istream&, TravelCompany&);

	void add(std::ofstream&);
	void read(std::ifstream&);
	static void create_file(std::ofstream&);

	void delete_data(std::ofstream&, std::ifstream&);
	void replacement(std::ofstream&, std::ifstream&, TravelCompany&);
	static void convert_to_txt(const std::string& filepath);

};